/***************************************************************************
 *   Copyright (C) 2003 - 2004 by Thomas Hasart
 *   thasart@gmx.de
 *
 *   Copyright (C) 2001-2002 by Bernd Gehrmann
 *   bernd@kdevelop.org
 *
 *   Copyright (C) 2002 by Jakob Simon-Gaarde
 *   jakob@jsg.dk
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version
 *
 ***************************************************************************/

#include "trollprojectpart.h"

#include <qdir.h>
#include <qfileinfo.h>
#include <qwhatsthis.h>
#include <kdeversion.h>
#include <kdebug.h>
#include <kdialogbase.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <qmessagebox.h>
#include <kdevgenericfactory.h>
#include <kaction.h>
#include <kparts/part.h>
#include <kprocess.h>
#include <makeoptionswidget.h>
#include <qregexp.h>


#include "domutil.h"
#include "kdevcore.h"
#include "kdevmainwindow.h"
#include "kdevmakefrontend.h"
#include "kdevappfrontend.h"
#include "kdevpartcontroller.h"
#include "trollprojectwidget.h"
#include "runoptionswidget.h"
#include "config.h"
#include "envvartools.h"
#include <kpopupmenu.h>


typedef KDevGenericFactory<TrollProjectPart> TrollProjectFactory;
static const KAboutData data("kdevtrollproject", I18N_NOOP("QMake Manager"), "1.0");
K_EXPORT_COMPONENT_FACTORY( libkdevtrollproject, TrollProjectFactory( &data ) )



//-------------------------------------------------------------------------------------------------------------------------------------------
TrollProjectPart::TrollProjectPart(QObject *parent, const char *name, const QStringList& args )
		: KDevProject("TrollProject", "trollproject", parent, name ? name : "TrollProjectPart")
{
	setInstance(TrollProjectFactory::instance());

	if ( args.count() == 1 && args[0] == "TMake" )
		m_tmakeProject = true;
	else
		m_tmakeProject = false;

	setXMLFile("kdevtrollproject.rc");

	m_executeAfterBuild = false;

	m_widget = new TrollProjectWidget(this);
	m_widget->setIcon(SmallIcon("make"));
	m_widget->setCaption(i18n("QMake Manager"));
	QWhatsThis::add(m_widget, i18n("<b>QMake manager</b><p>"
		                               "The QMake manager project tree consists of two parts. The 'overview' "
		                               "in the upper half shows the subprojects, each one having a "
		                               ".pro file. The 'details' view in the lower half shows the "
		                               "list of files for the active subproject selected in the overview."));

	initActions();
	m_widget->init(this);
	m_widget->setIcon(SmallIcon("qmakerun"));
	mainWindow()->embedSelectViewRight(m_widget, i18n("QMake Manager"), i18n("QMake manager"));



	connect( core(), SIGNAL(projectConfigWidget(KDialogBase*)),
	         this, SLOT(projectConfigWidget(KDialogBase*)) );

	connect( makeFrontend(), SIGNAL(commandFinished(const QString&)),
	         this, SLOT(slotCommandFinished(const QString&)) );

	m_qmakeHeader = i18n("# File generated by kdevelop's qmake manager. \n"
	                     "# ------------------------------------------- \n"
	                     "# Subdir relative project main directory: %s\n"
	                     "# Target is %s %s\n");

	m_availableQtDirList = availableQtDirList();
	m_defaultQtDir = ::getenv( "QTDIR" );
	if( m_defaultQtDir.isEmpty() && !m_availableQtDirList.isEmpty() )
		m_defaultQtDir = m_availableQtDirList.front();
}


//-------------------------------------------------------------------------------------------------------------------------------------------
TrollProjectPart::~TrollProjectPart()
{
	if (m_widget)
		mainWindow()->removeView(m_widget);
	delete m_widget;
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectPart::initActions()
{
	KAction *action;

	const QIconSet icon(SmallIcon("compfile"));
	action = new KAction( i18n("Compile &File"), "compfile", 0,
	                      m_widget, SLOT(slotMakeCommandProject()),
	                      actionCollection(),"build_compilefile"  );
	action->setToolTip(i18n("Compile file"));
	action->setWhatsThis(i18n("<b>Compile file</b><p>Runs <b>make filename.o</b> command from the directory where 'filename' is the name of currently opened file.<br>"
	                          "Environment variables and make arguments can be specified "
	                          "in the project settings dialog, <b>Make Options</b> tab."));



	action = new KAction( i18n("relink target"), "rebuild", 0,
	                      m_widget, SLOT(slotMakeCommandProject()),
	                      actionCollection(), "build_relink_target" );
	action->setToolTip(i18n("relink target"));
				  
				  
	action = new KAction( i18n("Execute Main Program"), "exec", SHIFT+Key_F9,
	                      m_widget, SLOT(slotExecuteProject()),
	                      actionCollection(), "build_execute_project" );
	action->setToolTip(i18n("Execute main program"));
	action->setWhatsThis(i18n("<b>Execute program</b><p>Executes the main program specified in project settings, <b>Run Options</b> tab."));






	action = new KAction( i18n("Execute Subproject"), "exec", 0,
	                      m_widget, SLOT(slotExecuteTarget()),
	                      actionCollection(), "build_execute_target" );
	action->setToolTip(i18n("Execute subproject"));
	action->setWhatsThis(i18n("<b>Execute subproject</b><p>Executes the target program for the currently selected subproject. "
	                          "This action is allowed only if a type of the subproject is 'application'. The type of the subproject can be "
	                          "defined in <b>Subproject Settings</b> dialog (open it from the subproject context menu)."));


	action = new KAction( i18n("Run qmake"), "qmakerun", 0,
	                      m_widget, SLOT(slotRunQMake()),
	                      actionCollection(), "build_run_qmake" );
	action->setWhatsThis( i18n("<b>Run qmake</b><p>Runs <b>qmake</b> from the selected subproject directory. This creates or regenerates Makefile."));


	action = new KAction( i18n("Subproject Settings"), "configure", 0,
	                      m_widget, SLOT(slotConfigureProject()),
	                      actionCollection(), "config_subproject" );
	action->setWhatsThis(i18n("<b>Subproject settings</b><p>Opens <b>QMake Subproject Configuration</b> dialog. "
	                          "It provides settings for:<br>subproject type and configuration,<br>include and library paths,<br>lists of dependencies and "
	                          "external libraries,<br>build order,<br>intermediate files locations,<br>compiler options."));


	//sources

	action = new KAction( i18n("Create new file"), "filenew", 0,
	                      m_widget, SLOT(slotCreateNewFile()),
	                      actionCollection(), "source_new_file" );
	action->setWhatsThis(i18n("<b>Create new file</b><p>Creates a new file and adds it to a currently selected group."));


	action = new KAction( i18n("Add existing files"), "fileimport", 0,
	                      m_widget, SLOT(slotAddFiles ()),
	                      actionCollection(), "source_add_files" );
	action->setWhatsThis(i18n("<b>Add existing files</b><p>Adds existing files to a currently selected group. It is "
	                          "possible to copy files to a current subproject directory, create symbolic links or "
	                          "add them with the relative path."));

	action = new KAction(i18n( "Remove file" ), "button_cancel", 0,
	                     m_widget, SLOT(slotRemoveFile()),
	                     actionCollection(), "source_remove_file" );
	action->setWhatsThis(i18n("<b>Remove file</b><p>Removes file from a current group. Does not remove file from disk."));


	action = new KAction( i18n( "Configure file" ), "configure_file", 0,
	                      m_widget, SLOT(slotConfigureFile()),
	                      actionCollection(), "source_config_file" );
	action->setWhatsThis(i18n("<b>Configure file</b><p>Opens <b>File Properties</b> dialog that allows a file to be excluded from specified scopes."));


	
	

	KActionMenu *actionMenu = new KActionMenu(i18n("add / remove..."),"",actionCollection(),"build_create_menu");
	action = new KAction( i18n("Create/Add Subproject..."), "folder_new", 0,
	                      m_widget, SLOT(slotAddSubdir()),
	                      actionCollection(), "config_add_subproject" );
	action->setWhatsThis( i18n("<b>Add subproject</b><p>Creates a <i>new</i> or adds an <i>existing</i> subproject to a currently selected subproject. "
	                           "This action is allowed only if a type of the subproject is 'subdirectories'. The type of the subproject can be "
	                           "defined in <b>Subproject Settings</b> dialog (open it from the subproject context menu)."));
	action->plug(actionMenu->popupMenu());
	
	action = new KAction( i18n("Create Scope..."), "qmake_scopenew", 0,
	                      m_widget, SLOT(slotCreateScope()),
	                      actionCollection(), "config_create_scope" );
	action->setWhatsThis(i18n("<b>Create scope</b><p>Creates QMake scope in the project file of the currently selected subproject."));
	action->plug(actionMenu->popupMenu());
	
	actionMenu->popupMenu()->insertSeparator();

	action = new KAction( i18n("Remove Subproject..."), "remove_subdir", 0,
	                      m_widget, SLOT(slotRemoveSubproject()),
	                      actionCollection(), "config_remove_subproject" );
	action->setWhatsThis( i18n("<b>Remove subproject</b><p>Removes currently selected subproject. Does not delete any file from disk. Deleted subproject can be later added by calling 'Add Subproject' action."));
	action->plug(actionMenu->popupMenu());
	
	action = new KAction( i18n("Remove Scope"), "editdelete", 0,
	                      m_widget, SLOT(slotRemoveScope()),
	                      actionCollection(), "config_remove_scope" );
	action->setWhatsThis( i18n("<b>Remove Scope</b><p>Removes currently selected scope."));
	action->plug(actionMenu->popupMenu());

		
		
		
	actionMenu = new KActionMenu(i18n("set build mode..."),"",actionCollection(),"build_buidmode_menu");
	action = new KAction( i18n( "Release (this)" ), "release", 0, m_widget, SLOT(slotBuildModeRelease()),
	                      actionCollection(), "buildmode_release" );
	action->plug(actionMenu->popupMenu());
	action = new KAction( i18n( "Debug (this)" ), "debug", 0, m_widget, SLOT(slotBuildModeDebug()),
	                      actionCollection(), "buildmode_debug" );
	action->plug(actionMenu->popupMenu());
	actionMenu->popupMenu()->insertSeparator();
	action = new KAction( i18n( "Release (all sub prj)" ), "release", 0, m_widget, SLOT(slotBuildModeReleaseAll()),
	                      actionCollection(), "buildmode_release_all" );
	action->plug(actionMenu->popupMenu());
	action = new KAction( i18n( "Debug (all sub prj)" ), "debug", 0, m_widget, SLOT(slotBuildModeDebugAll()),
	                      actionCollection(), "buildmode_debug_all" );
	action->plug(actionMenu->popupMenu());


	action = new KAction( i18n( "Subclass Widget..." ), "qmake_subclass.png", 0, m_widget, SLOT(slotSubclassWidget()),
	                      actionCollection(), "file_subclass_ui" );
	action->setWhatsThis(i18n("<b>Subclass widget</b><p>Launches <b>Subclassing</b> wizard. "
	                          "It allows to create a subclass from the class defined in .ui file. "
	                          "There is also possibility to implement slots and functions defined in the base class."));

	action = new KAction( i18n( "Open ui.h File" ), "qmake_ui_h.png", 0, m_widget, SLOT(slotFileOpenUiHeader()),
	                      actionCollection(), "file_open_ui_header" );
	action->setWhatsThis(i18n("<b>List of subclasses</b><p>Shows subclasses list editor. "
	                          "There is possibility to add or remove subclasses from the list."));

	action = new KAction( i18n( "Edit ui-Subclass..." ), "qmake_subclass.png", 0, m_widget, SLOT(slotFileEditUi()),
	                      actionCollection(), "file_subclass_ui_edit" );
	action->setWhatsThis( i18n("<b>Edit ui-subclass</b><p>Launches <b>Subclassing</b> wizard "
	                           "and prompts to implement missing in childclass slots and functions."));





	action = new KAction( i18n( "Update Translation Files" ), "konsole", 0, m_widget, SLOT(slotFileEditUi()),
	                      actionCollection(), "file_update_translation" );
	action->setWhatsThis(i18n("<b>Update Translation Files</b><p>Runs <b>lupdate</b> command from the current subproject directory. It collects translatable "
	                          "messages and saves them into translation files."));

	action = new KAction( i18n("Release Binary Translations" ), "konsole", 0, m_widget, SLOT(slotFileEditUi()),
	                      actionCollection(), "file_release_translation" );
	action->setWhatsThis(i18n("<b>Release Binary Translations</b><p>Runs <b>lrelease</b> command from the current subproject directory. It creates binary "
	                          "translation files that are ready to be loaded at program execution."));

				  
				  

	actionMenu = new KActionMenu(i18n("make"),"",actionCollection(),"build_make_menu");

	action = new KAction( i18n("make target"), "make_kdevelop", Key_F7,
	                      m_widget, SLOT(slotMakeCommandProject()),
	                      actionCollection(), "build_build_target" );
	action->setToolTip(i18n("Build subproject"));
	action->setWhatsThis(i18n("<b>Build subproject</b><p>Runs <b>make</b> from the current subproject directory. "
	                          "Current subproject is a subproject selected in <b>QMake manager</b> 'overview' window.<br>"
	                          "Environment variables and make arguments can be specified "
	                          "in the project settings dialog, <b>Make Options</b> tab."));
	action->plug(actionMenu->popupMenu());
	
	action = new KAction( i18n("make rebuild"), "rebuild", 0, m_widget, SLOT(slotMakeCommandProject()),
				actionCollection(),"build_rebuild_target"  );
	action->setToolTip(i18n("Rebuild subproject"));
	action->setWhatsThis(i18n("<b>Rebuild subproject</b><p>Runs <b>make clean</b> and then <b>make</b> from the current subproject directory. "
	                          "Current subproject is a subproject selected in <b>QMake manager</b> 'overview' window.<br>"
	                          "Environment variables and make arguments can be specified "
	                          "in the project settings dialog, <b>Make Options</b> tab."));
	action->plug(actionMenu->popupMenu());
	
	action = new KAction( i18n("make moccables"), "make_kdevelop",0,
	                      m_widget, SLOT(slotMakeCommandProject()),
	                      actionCollection(), "build_mocables_target" );
	action->setToolTip(i18n("Build moc's target"));
	action->plug(actionMenu->popupMenu());
	
	action = new KAction( i18n("make uicables"), "make_kdevelop", 0,
	                      m_widget, SLOT(slotMakeCommandProject()),
	                      actionCollection(), "build_uicables_target" );
	action->setToolTip(i18n("Build ui's target"));
	action->plug(actionMenu->popupMenu());
	
	actionMenu->popupMenu()->insertSeparator();

		
	action = new KAction( i18n("make clean"), 0, m_widget, SLOT(slotMakeCommandProject()),actionCollection()
				, "build_clean_target" );
	action->setToolTip(i18n("Clean subproject"));
	action->setWhatsThis(i18n("<b>Clean subproject</b><p>Runs <b>make clean</b> from the current subproject directory. "
	                          "Current subproject is a subproject selected in <b>QMake manager</b> 'overview' window.<br>"
	                          "Environment variables and make arguments can be specified "
	                          "in the project settings dialog, <b>Make Options</b> tab."));
	action->plug(actionMenu->popupMenu());
	
	action = new KAction( i18n("make distclean"), 0,
	                      m_widget, SLOT(slotMakeCommandProject()),
	                      actionCollection(), "build_distclean_target" );
	action->setToolTip(i18n("distclean target"));
	action->plug(actionMenu->popupMenu());

	
	action = new KAction( i18n("make uiclean"), 0,
	                      m_widget, SLOT(slotMakeCommandProject()),
	                      actionCollection(), "build_uiclean_target" );
	action->setToolTip(i18n("uiclean project"));
	action->plug(actionMenu->popupMenu());

	action = new KAction( i18n("make mocclean"), 0,
	                      m_widget, SLOT(slotMakeCommandProject()),
	                      actionCollection(), "build_mocclean_target" );
	action->setToolTip(i18n("mocclean target"));
	action->plug(actionMenu->popupMenu());

	action = new KAction( i18n("make lexclean"), 0,
	                      m_widget, SLOT(slotMakeCommandProject()),
	                      actionCollection(), "build_lexclean_target" );
	action->setToolTip(i18n("lexclean target"));
	action->plug(actionMenu->popupMenu());

	action = new KAction( i18n("make yaccclean"), 0,
	                      m_widget, SLOT(slotMakeCommandProject()),
	                      actionCollection(), "build_yaccclean_target" );
	action->setToolTip(i18n("caccclean target"));
	action->plug(actionMenu->popupMenu());
	
	actionMenu->popupMenu()->insertSeparator();
	
	action = new KAction( i18n("make install"), 0, m_widget, SLOT(slotMakeCommandProject()),
	                      actionCollection(), "build_install_target" );
	action->setToolTip(i18n("install target"));
	action->plug(actionMenu->popupMenu());

	
	//############################################################################################################
	actionMenu = new KActionMenu(i18n("make (target only)"),"",actionCollection(),"build_make_menu_targetonly");
		

	action = new KAction( i18n("make target"), "make_kdevelop", Key_F7,
	                      m_widget, SLOT(slotMakeCommandProject()),
	                      actionCollection(), "build_build_targetonly" );
	action->setToolTip(i18n("Build subproject"));
	action->setWhatsThis(i18n("<b>Build subproject</b><p>Runs <b>make</b> from the current subproject directory. "
	                          "Current subproject is a subproject selected in <b>QMake manager</b> 'overview' window.<br>"
	                          "Environment variables and make arguments can be specified "
	                          "in the project settings dialog, <b>Make Options</b> tab."));
	action->plug(actionMenu->popupMenu());

	
	
	action = new KAction( i18n("make rebuild"), "rebuild", 0, m_widget, SLOT(slotMakeCommandProject()),
				actionCollection(),"build_rebuild_targetonly"  );
	action->setToolTip(i18n("Rebuild subproject"));
	action->setWhatsThis(i18n("<b>Rebuild subproject</b><p>Runs <b>make clean</b> and then <b>make</b> from the current subproject directory. "
	                          "Current subproject is a subproject selected in <b>QMake manager</b> 'overview' window.<br>"
	                          "Environment variables and make arguments can be specified "
	                          "in the project settings dialog, <b>Make Options</b> tab."));
	action->plug(actionMenu->popupMenu());
	
	actionMenu->popupMenu()->insertSeparator();
	
	actionCollection()->action("build_install_target")->plug(actionMenu->popupMenu());

	
	//############################################################################################################
	actionMenu = new KActionMenu(i18n("make (complete project)"),"",actionCollection(),"build_make_menu_project");
	action = new KAction( i18n("make project"), "make_kdevelop", Key_F8,
	                      m_widget, SLOT(slotMakeCommandProject()),
	                      actionCollection(), "build_build_project" );
	action->setToolTip(i18n("Build project"));
	action->setWhatsThis(i18n("<b>Build project</b><p>Runs <b>make</b> from the project directory.<br>"
	                          "Environment variables and make arguments can be specified "
	                          "in the project settings dialog, <b>Make Options</b> tab."));
	action->plug(actionMenu->popupMenu());
	
	action = new KAction( i18n("make rebuild"),"rebuild" , 0,
	                      m_widget, SLOT(slotMakeCommandProject()),
	                      actionCollection(),"build_rebuild_project"  );
	action->setToolTip(i18n("Rebuild project"));
	action->setWhatsThis(i18n("<b>Rebuild project</b><p>Runs <b>make clean</b> and then <b>make</b> from the project directory.<br>"
	                          "Environment variables and make arguments can be specified "
	                          "in the project settings dialog, <b>Make Options</b> tab."));
	action->plug(actionMenu->popupMenu());

	actionMenu->popupMenu()->insertSeparator();
	
	action = new KAction( i18n("make clean"), 0, m_widget, SLOT(slotMakeCommandProject()),
	                      actionCollection(), "build_clean_project" );
	action->setToolTip(i18n("Clean project"));
	action->setWhatsThis(i18n("<b>Clean project</b><p>Runs <b>make clean</b> command from the project directory.<br>"
	                          "Environment variables and make arguments can be specified "
	                          "in the project settings dialog, <b>Make Options</b> tab."));
	action->plug(actionMenu->popupMenu());

	action = new KAction( i18n("make distclean"), 0, m_widget, SLOT(slotMakeCommandProject()),
	                      actionCollection(), "build_distclean_project" );
	action->setToolTip(i18n("Clean project"));
	action->setWhatsThis(i18n("<b>DistClean project</b><p>Runs <b>make distclean</b> command from the project directory.<br>"
	                          "Environment variables and make arguments can be specified "
	                          "in the project settings dialog, <b>Make Options</b> tab."));
	action->plug(actionMenu->popupMenu());
	actionMenu->popupMenu()->insertSeparator();
	
	action = new KAction( i18n("make install"), 0, m_widget, SLOT(slotMakeCommandProject()),
	                      actionCollection(), "build_install_project" );
	action->setToolTip(i18n("install project"));
	action->plug(actionMenu->popupMenu());
	
	
	//############################################################################################################
	action = new KAction( i18n("add install group"), "", 0,
	                      m_widget, SLOT(slotAddInstallRoot()),
	                      actionCollection(), "add_install_root" );
	action->setToolTip(i18n("add install group"));

	action = new KAction( i18n("set install path"), "", 0,
	                      m_widget, SLOT(slotSetInstallPath()),
	                      actionCollection(), "set_install_path" );
	action->setToolTip(i18n("set install path"));

	action = new KAction( i18n("add install files"), "", 0,
	                      m_widget, SLOT(slotAddInstallFiles()),
	                      actionCollection(), "add_install_files" );
	action->setToolTip(i18n("add install files"));

	action = new KAction( i18n("remove install object"), "", 0,
	                      m_widget, SLOT(slotRemoveInstallRoot()),
	                      actionCollection(), "remove_install_root" );
	action->setToolTip(i18n("remove install object"));

	action = new KAction( i18n("set main target"), "", 0,
	                      m_widget, SLOT(slotSetMainTarget()),
	                      actionCollection(), "set_main_target" );
	action->setToolTip(i18n("set main target"));

	

	
	//############################################################################################################
	//############################################################################################################
	actionMenu = new KActionMenu(i18n("extras"),"",actionCollection(),"build_extra_menu");
	action = new KAction( i18n("create pch symlinks"), "", 0,
	                      m_widget, SLOT(slotCreatePchDirLinks()),
	                      actionCollection(), "create_pch_links" );
	action->setToolTip(i18n("create pch symlinks"));
	action->plug(actionMenu->popupMenu());
	
	
	action = new KAction( i18n("apply build mode to all projects"), "", 0,
	                      m_widget, SLOT(slotApplyToSubProjects()),
	                      actionCollection(), "apply_build_mode" );
	action->setToolTip(i18n("apply build mode to all projects"));
	action->plug(actionMenu->popupMenu());

		
	action = new KAction( i18n("apply debug cxx flags to all projects"), "", 0,
	                      m_widget, SLOT(slotApplyToSubProjects()),
	                      actionCollection(), "apply_debug_cxx" );
	action->setToolTip(i18n("apply debug cxx flags to all projects"));
	action->plug(actionMenu->popupMenu());

		
	action = new KAction( i18n("apply release cxx flags to all projects"), "", 0,
	                      m_widget, SLOT(slotApplyToSubProjects()),
	                      actionCollection(), "apply_release_cxx" );
	action->setToolTip(i18n("apply release cxx flags to all projects"));
	action->plug(actionMenu->popupMenu());

	action = new KAction( i18n("apply cxx flags to all projects"), "", 0,
	                      m_widget, SLOT(slotApplyToSubProjects()),
	                      actionCollection(), "apply_cxx" );
	action->setToolTip(i18n("apply cxx flags to all projects"));
	action->plug(actionMenu->popupMenu());

	action = new KAction( i18n("apply locations to all projects"), "", 0,
	                      m_widget, SLOT(slotApplyToSubProjects()),
	                      actionCollection(), "apply_locations" );
	action->setToolTip(i18n("apply locations to all projects"));
	action->plug(actionMenu->popupMenu());
	

	action = new KAction( i18n("apply pch include to all projects"), "", 0,
	                      m_widget, SLOT(slotApplyToSubProjects()),
	                      actionCollection(), "apply_pch_include" );
	action->setToolTip(i18n("apply pch include to all projects"));
	action->plug(actionMenu->popupMenu());
	
	action = new KAction( i18n("apply use pch ro all projects"), "", 0,
	                      m_widget, SLOT(slotApplyToSubProjects()),
	                      actionCollection(), "apply_use_pch" );
	action->setToolTip(i18n("apply use pch to all projects"));
	action->plug(actionMenu->popupMenu());
	
	action = new KAction( i18n("apply requirements to all projects"), "", 0,
	                      m_widget, SLOT(slotApplyToSubProjects()),
	                      actionCollection(), "apply_requirements" );
	action->setToolTip(i18n("apply requirements ro all projects"));
	action->plug(actionMenu->popupMenu());

	action = new KAction( i18n("apply all settings to all projects"), "", 0,
	                      m_widget, SLOT(slotApplyToSubProjects()),
	                      actionCollection(), "apply_all" );
	action->setToolTip(i18n("apply all settings to all projects"));
	action->plug(actionMenu->popupMenu());
}


//-------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectPart::makeEnvironment()
{
	// Get the make environment variables pairs into the environstr string
	// in the form of: "ENV_VARIABLE=ENV_VALUE"
	// Note that we quote the variable value due to the possibility of
	// embedded spaces
	DomUtil::PairList envvars =
	    DomUtil::readPairListEntry(*projectDom(), "/kdevtrollproject/make/envvars", "envvar", "name", "value");

	QString environstr;
	DomUtil::PairList::ConstIterator it;
	bool hasQtDir = false;
	for (it = envvars.begin(); it != envvars.end(); ++it)
	{
		if( (*it).first == "QTDIR" )
			hasQtDir = true;

		environstr += (*it).first;
		environstr += "=";
		/*
		#if (KDE_VERSION > 305)
		        environstr += KProcess::quote((*it).second);
		#else
		        environstr += KShellProcess::quote((*it).second);
		#endif
		*/
		environstr += EnvVarTools::quote((*it).second);
		environstr += " ";
	}

	if( !hasQtDir && !m_defaultQtDir.isEmpty() )
	{
		environstr += QString( "QTDIR=" ) + EnvVarTools::quote( m_defaultQtDir ) + QString( " " );
	}

	return environstr;
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectPart::projectConfigWidget(KDialogBase *dlg)
{
	QVBox *vbox;
	vbox = dlg->addVBoxPage(i18n("Run Options"));
	RunOptionsWidget *optdlg = new RunOptionsWidget(*projectDom(), "/kdevtrollproject", buildDirectory(), vbox);

	vbox = dlg->addVBoxPage(i18n("Make Options"));
	MakeOptionsWidget *w4 = new MakeOptionsWidget(*projectDom(), "/kdevtrollproject", vbox);
	connect( dlg, SIGNAL(okClicked()), w4, SLOT(accept()) );

	connect( dlg, SIGNAL(okClicked()), optdlg, SLOT(accept()) );
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectPart::openProject(const QString &dirName, const QString &projectName)
{
	m_widget->openProject(dirName);
	m_projectName = projectName;

	QDomDocument &dom = *projectDom();
	// Set the default directory radio to "executable"
	if (DomUtil::readEntry(dom, "/kdevtrollproject/run/directoryradio") == "" )
	{
		DomUtil::writeEntry(dom, "/kdevtrollproject/run/directoryradio", "executable");
	}

	KDevProject::openProject( dirName, projectName );
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectPart::closeProject()
{
	m_widget->closeProject();
}


//-------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectPart::projectDirectory() const
{
	return m_widget->projectDirectory();
}


//-------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectPart::buildDirectory() const
{
	return m_widget->projectDirectory();
}


//-------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectPart::projectName() const
{
	return m_projectName;
}


//-------------------------------------------------------------------------------------------------------------------------------------------
/** Retuns a PairList with the run environment variables */
DomUtil::PairList TrollProjectPart::runEnvironmentVars() const
{
	return DomUtil::readPairListEntry(*projectDom(), "/kdevtrollproject/run/envvars", "envvar", "name", "value");
}



//-------------------------------------------------------------------------------------------------------------------------------------------
/** Retuns the currently selected run directory
  * The returned string can be:
  *   if run/directoryradio == executable
  *        The directory where the executable is
  *   if run/directoryradio == build
  *        The directory where the executable is relative to build directory
  *   if run/directoryradio == custom
  *        The custom directory absolute path
  */
QString TrollProjectPart::runDirectory() const
{
	QDomDocument &dom = *projectDom();

	QString directoryRadioString = DomUtil::readEntry(dom, "/kdevtrollproject/run/directoryradio");
	QString DomMainProgram = DomUtil::readEntry(dom, "/kdevtrollproject/run/mainprogram");

	if ( directoryRadioString == "build" )
		return buildDirectory();

	if ( directoryRadioString == "custom" )
		return DomUtil::readEntry(dom, "/kdevtrollproject/run/customdirectory");

	int pos = DomMainProgram.findRev('/');
	if (pos != -1)
		return buildDirectory() + "/" + DomMainProgram.left(pos);

	if ( DomMainProgram.isEmpty() )
	{
		return m_widget->subprojectDirectory();
	}
	return buildDirectory() + "/" + DomMainProgram;

}



//-------------------------------------------------------------------------------------------------------------------------------------------
/** Retuns the currently selected main program
  * The returned string can be:
  *   if run/directoryradio == executable
  *        The executable name
  *   if run/directoryradio == build
  *        The path to executable relative to build directory
  *   if run/directoryradio == custom or relative == false
  *        The absolute path to executable
  */
QString TrollProjectPart::mainProgram(bool relative) const
{
	QDomDocument &dom = *projectDom();

	QString directoryRadioString = DomUtil::readEntry(dom, "/kdevtrollproject/run/directoryradio");
	QString DomMainProgram = DomUtil::readEntry(dom, "/kdevtrollproject/run/mainprogram");

	if ( directoryRadioString == "custom" )
		return DomMainProgram;

	if ( relative == false && !DomMainProgram.isEmpty() )
		return buildDirectory() + "/" + DomMainProgram;

	if ( directoryRadioString == "executable" )
	{
		int pos = DomMainProgram.findRev('/');
		if (pos != -1)
			return DomMainProgram.mid(pos+1);

		if ( DomMainProgram.isEmpty() )
		{
			return runDirectory() + "/" + m_widget->getCurrentOutputFilename();
		}
		return DomMainProgram;
	}
	else
		return DomMainProgram;
}








//-------------------------------------------------------------------------------------------------------------------------------------------
/** Retuns a QString with the run command line arguments */
QString TrollProjectPart::runArguments() const
{
	return DomUtil::readEntry(*projectDom(), "/kdevtrollproject/run/programargs");
}


//-------------------------------------------------------------------------------------------------------------------------------------------
QString TrollProjectPart::activeDirectory() const
{
	QDomDocument &dom = *projectDom();

	return DomUtil::readEntry(dom, "/kdevtrollproject/general/activedir");
}


//-------------------------------------------------------------------------------------------------------------------------------------------
QStringList TrollProjectPart::allFiles() const
{
	return m_widget->allFiles();
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectPart::addFile(const QString &fileName)
{
	QStringList fileList;
	fileList.append ( fileName );

	this->addFiles ( fileList );
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectPart::addFiles ( const QStringList &fileList )
{
	QStringList files = fileList;
	for (QStringList::iterator it = files.begin(); it != files.end(); ++it)
		//        if (!(*it).contains(projectDirectory()))
		*it = projectDirectory() + "/" + (*it);

	m_widget->addFiles(files);

	//	emit addedFilesToProject ( files );
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectPart::removeFile(const QString & /* fileName */)
{
	/// \FIXME
	/*	QStringList fileList;
		fileList.append ( fileName );
		
		this->removeFiles ( fileList );*/
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectPart::removeFiles ( const QStringList& fileList )
{
	/// \FIXME missing remove files functionality
	// 	QStringList::ConstIterator it;
	//
	// 	it = fileList.begin();
	//
	// 	for ( ; it != fileList.end(); ++it )
	// 	{
	// 		FIXME
	// 	}

	emit removedFilesFromProject ( fileList );
}



//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectPart::startQMakeCommand(const QString &dir)
{
	QFileInfo fi(dir);

	QString cmdline = QString::fromLatin1( isTMakeProject() ? "tmake " : "qmake " );
	//    cmdline += fi.baseName() + ".pro";
	QDir d(dir);
	QStringList l = d.entryList("*.pro");

	cmdline += l.count()?l[0]:(fi.baseName() + ".pro");

	//    cmdline += QString::fromLatin1( " -o Makefile" );

	QString dircmd = "cd ";
	dircmd += dir;
	dircmd += " && ";

	cmdline.prepend(makeEnvironment());
	makeFrontend()->queueCommand(dir, dircmd + cmdline);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectPart::queueCmd(const QString &dir, const QString &cmd)
{
	makeFrontend()->queueCommand(dir, cmd);
}


//-------------------------------------------------------------------------------------------------------------------------------------------
void TrollProjectPart::slotCommandFinished( const QString& command )
{
	Q_UNUSED( command );

	if( m_buildCommand != command )
		return;

	m_buildCommand = QString::null;

	m_timestamp.clear();
	QStringList fileList = allFiles();
	QStringList::Iterator it = fileList.begin();
	while( it != fileList.end() )
	{
		QString fileName = *it;
		++it;

		m_timestamp[ fileName ] = QFileInfo( projectDirectory(), fileName ).lastModified();
	}

	emit projectCompiled();

	if( m_executeAfterBuild )
	{
		m_widget->slotExecuteProject();
		m_executeAfterBuild = false;
	}
}


//-------------------------------------------------------------------------------------------------------------------------------------------
bool TrollProjectPart::isDirty()
{
	QStringList fileList = allFiles();
	QStringList::Iterator it = fileList.begin();
	while( it != fileList.end() )
	{
		QString fileName = *it;
		++it;

		QMap<QString, QDateTime>::Iterator it = m_timestamp.find( fileName );
		QDateTime t = QFileInfo( projectDirectory(), fileName ).lastModified();
		if( it == m_timestamp.end() || *it != t )
		{
			return true;
		}
	}

	return false;
}


//-------------------------------------------------------------------------------------------------------------------------------------------
KDevProject::Options TrollProjectPart::options( )
{
	return UsesQMakeBuildSystem;
}


//-------------------------------------------------------------------------------------------------------------------------------------------
bool TrollProjectPart::isValidQtDir( const QString& path ) const
{
	return QFile::exists( path + "/include/qt.h" );
}


//-------------------------------------------------------------------------------------------------------------------------------------------
QStringList TrollProjectPart::availableQtDirList() const
{
	QStringList qtdirs, lst;
	qtdirs.push_back( ::getenv("QTDIR") );
	qtdirs.push_back( "/usr/lib/qt3" );
	qtdirs.push_back( "/usr/lib/qt" );
	qtdirs.push_back( "/usr/share/qt3" );

	for( QStringList::Iterator it=qtdirs.begin(); it!=qtdirs.end(); ++it )
	{
		QString qtdir = *it;
		if( !qtdir.isEmpty() && isValidQtDir(qtdir) )
			lst.push_back( qtdir );
	}
	return lst;
}


//-------------------------------------------------------------------------------------------------------------------------------------------
/*!
    \fn TrollProjectPart::distFiles() const
 */
QStringList TrollProjectPart::distFiles() const
{
	QStringList sourceList = allFiles();
	// Scan current source directory for any .pro files.
	QString projectDir = projectDirectory();
	QDir dir(projectDir);
	QStringList files = dir.entryList( "*.pro *.PRO Makefile");
	return sourceList + files;
}

#include "trollprojectpart.moc"
